﻿using Syncfusion.SfChart.XForms;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace SimpleSample
{
    public partial class MainPage : ContentPage
    {
        ViewModel viewModel;

        public MainPage()
        {
            InitializeComponent();
            viewModel = new ViewModel();
            this.BindingContext = viewModel;

            series2.StrokeRegion = new DateTime(2006, 01, 01);
            series1.StrokeDashArray = new double[] { 5, 3 };
        }
    }

    public class ViewModel
    {
        public ObservableCollection<Model> LineSeriesData { get; set; }
        public ViewModel()
        {
            LineSeriesData = new ObservableCollection<Model>();
            Random rand = new Random();

            var dateTime = new DateTime(2002, 01, 01);
            for (int i = 0; i < 10; i++)
            {
                LineSeriesData.Add(new Model(dateTime, rand.Next(20, 50)));
                dateTime = dateTime.AddYears(1);
            }
        }
    }

    public class Model
    {
        public DateTime Date { get; set; }

        public double Value { get; set; }
        public Model(DateTime date, double value)
        {
            Date = date;
            Value = value;
        }
    }

    public class FastLineSeriesExt : FastLineSeries
    {
        /// <summary>
        /// Gets or sets the data source to the chart. This is a bindable property.
        /// </summary> 
        public static readonly BindableProperty DataSourceProperty =
            BindableProperty.Create("DataSource", typeof(IEnumerable), typeof(FastLineSeriesExt), null, BindingMode.Default, null, OnDataSourceChanged);

        /// <summary>
        /// Gets or sets the Stroke started region. This is a bindable property.
        /// </summary> 
        public static readonly BindableProperty StrokeRegionProperty =
            BindableProperty.Create("StrokeRegion", typeof(DateTime?), typeof(FastLineSeriesExt), null, BindingMode.Default, null,OnStrokeRegionChanged);
        
        public IEnumerable DataSource
        {
            get { return (IEnumerable)GetValue(DataSourceProperty); }
            set { SetValue(DataSourceProperty, value); }
        }

        public DateTime? StrokeRegion
        {
            get { return (DateTime?)GetValue(StrokeRegionProperty); }
            set { SetValue(StrokeRegionProperty, value); }
        }

        private static void OnDataSourceChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var fastLineSeries = bindable as FastLineSeriesExt;
            SetItemSource(fastLineSeries);
        }

        private static void OnStrokeRegionChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var fastLineSeries = bindable as FastLineSeriesExt;
            SetItemSource(fastLineSeries);
        }

        private static void SetItemSource(FastLineSeriesExt fastLineSeries)
        { 
            if (fastLineSeries == null || fastLineSeries.DataSource == null)
                return;

            var data = fastLineSeries.DataSource as ObservableCollection<Model>;
            fastLineSeries.ItemsSource = data?.Where(a => a.Date <= fastLineSeries.StrokeRegion).ToList();
        }
    }
}
